@@ -71,9 +71,12 @@ typedef struct {
   /* An EmpathyAccountWidget can be used to either create an account or
    * modify it. When we are creating an account, this member is set to TRUE */
   gboolean creating_account;
-  /* If we are creating a new account, this member is set to TRUE once the
-   * account has been created */
-  gboolean account_created;
+
+  /* if TRUE, the GTK+ destroy signal has been fired and so the widgets
+   * embedded in this account widget can't be used any more
+   * workaround because some async callbacks can be called after the
+   * widget has been destroyed */
+  gboolean destroyed;
 
   TpAccountManager *account_manager;
 
@@ -110,8 +113,6 @@ account_widget_set_control_buttons_sensitivity (EmpathyAccountWidget *self,
       gtk_widget_set_sensitive (priv->apply_button, sensitive);
       gtk_widget_set_sensitive (
           priv->cancel_button, sensitive || priv->creating_account);
-
-      priv->contains_pending_changes = sensitive;
     }
 }
 
@@ -166,8 +167,12 @@ static void
 account_widget_entry_changed_cb (GtkEditable *entry,
     EmpathyAccountWidget *self)
 {
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
   account_widget_entry_changed_common (self, GTK_ENTRY (entry), FALSE);
   account_widget_handle_control_buttons_sensitivity (self);
+
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -209,6 +214,7 @@ account_widget_int_changed_cb (GtkWidget *widget,
     }
 
   account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -241,6 +247,7 @@ account_widget_checkbutton_toggled_cb (GtkWidget *widget,
     }
 
   account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -258,6 +265,7 @@ account_widget_forget_clicked_cb (GtkWidget *button,
   gtk_entry_set_text (GTK_ENTRY (priv->entry_password), "");
 
   account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -269,6 +277,8 @@ account_widget_password_changed_cb (GtkWidget *entry,
 
   str = gtk_entry_get_text (GTK_ENTRY (entry));
   gtk_widget_set_sensitive (priv->button_forget, !EMP_STR_EMPTY (str));
+
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -294,6 +304,8 @@ account_widget_jabber_ssl_toggled_cb (GtkWidget *checkbutton_ssl,
     }
 
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (priv->spinbutton_port), port);
+
+  priv->contains_pending_changes = TRUE;
 }
 
 static void
@@ -333,6 +345,7 @@ account_widget_combobox_changed_cb (GtkWidget *widget,
     }
 
   account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
 }
 
 void
@@ -654,19 +667,41 @@ account_widget_account_enabled_cb (GObject *source_object,
   TpAccount *account = TP_ACCOUNT (source_object);
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
+  TpConnectionPresenceType presence;
+  gchar *message = NULL;
+  gchar *status = NULL;
 
   tp_account_set_enabled_finish (account, res, &error);
 
   if (error != NULL)
     {
-      DEBUG ("Could not automatically enable new account: %s", error->message);
+      DEBUG ("Could not enable the account: %s", error->message);
       g_error_free (error);
     }
   else
     {
-      priv->account_created = TRUE;
-      g_signal_emit (widget, signals[ACCOUNT_CREATED], 0);
+      /* only force presence if presence was offline, unknown or unset */
+      presence = tp_account_get_requested_presence (account, NULL, NULL);
+      switch (presence)
+        {
+        case TP_CONNECTION_PRESENCE_TYPE_OFFLINE:
+        case TP_CONNECTION_PRESENCE_TYPE_UNKNOWN:
+        case TP_CONNECTION_PRESENCE_TYPE_UNSET:
+          presence = tp_account_manager_get_most_available_presence (
+              priv->account_manager, &status, &message);
+          tp_account_request_presence_async (account, presence,
+              status, NULL, NULL, NULL);
+          break;
+        default:
+          /* do nothing if the presence is not offline */
+          break;
+        }
     }
+
+  /* unref widget - part of the workaround */
+  g_object_unref (widget);
+  g_free (message);
+  g_free (status);
 }
 
 static void
@@ -696,8 +731,13 @@ account_widget_applied_cb (GObject *source_object,
       if (priv->creating_account)
         {
           /* By default, when an account is created, we enable it. */
+
+          /* workaround to keep widget alive during async call */
+          g_object_ref (widget);
+
           tp_account_set_enabled_async (account, TRUE,
               account_widget_account_enabled_cb, widget);
+          g_signal_emit (widget, signals[ACCOUNT_CREATED], 0);
         }
       else if (priv->enabled_checkbox != NULL)
         {
@@ -722,7 +762,13 @@ account_widget_applied_cb (GObject *source_object,
         }
     }
 
-  account_widget_set_control_buttons_sensitivity (widget, FALSE);
+  if (!priv->destroyed)
+    account_widget_set_control_buttons_sensitivity (widget, FALSE);
+
+  priv->contains_pending_changes = FALSE;
+
+  /* unref the widget - part of the workaround */
+  g_object_unref (widget);
 }
 
 static void
@@ -731,6 +777,8 @@ account_widget_apply_clicked_cb (GtkWidget *button,
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
 
+  /* workaround to keep widget alive during async call */
+  g_object_ref (self);
   empathy_account_settings_apply_async (priv->settings,
       account_widget_applied_cb, self);
 }
@@ -1126,6 +1174,10 @@ static void
 account_widget_destroy_cb (GtkWidget *widget,
     EmpathyAccountWidget *self)
 {
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  /* set the destroyed flag - workaround */
+  priv->destroyed = TRUE;
+
   g_object_unref (self);
 }
 
@@ -1172,7 +1224,10 @@ account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
   account = empathy_account_settings_get_account (priv->settings);
 
   /* Enable the account according to the value of the "Enabled" checkbox */
-  tp_account_set_enabled_async (account, state, NULL, NULL);
+  /* workaround to keep widget alive during async call */
+  g_object_ref (user_data);
+  tp_account_set_enabled_async (account, state,
+      account_widget_account_enabled_cb, user_data);
 }
 
 static void
@@ -1374,6 +1429,12 @@ do_constructed (GObject *obj)
           self);
     }
 
+  /* dup and init the account-manager */
+  priv->account_manager = tp_account_manager_dup ();
+
+  tp_account_manager_prepare_async (priv->account_manager, NULL,
+      account_manager_ready_cb, self);
+
   /* handle apply and cancel button */
   if (!priv->simple)
     {
@@ -1383,18 +1444,13 @@ do_constructed (GObject *obj)
 
       if (priv->creating_account)
         {
-          priv->account_manager = tp_account_manager_dup ();
+          /* Assumre we are offline, display a Save button. We'll update
+           * it once the account manager is ready if needed */
+          priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
 
           empathy_signal_connect_weak (priv->account_manager,
               "most-available-presence-changed",
               G_CALLBACK (presence_changed_cb), obj);
-
-          tp_account_manager_prepare_async (priv->account_manager, NULL,
-              account_manager_ready_cb, self);
-
-          /* Assumre we are offline, display a Save button. We'll update
-           * it once the account manager is ready if needed */
-          priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
         }
       else
         {
@@ -1650,11 +1706,6 @@ empathy_account_widget_contains_pending_changes (EmpathyAccountWidget *widget)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
 
-  if (priv->creating_account && !priv->account_created)
-    /* We always want to warn the user if he's in the process of creating a
-     * new account which hasn't been actually created yet. */
-    return TRUE;
-
   return priv->contains_pending_changes;
 }
 
