@@ -60,11 +60,13 @@ typedef struct {
 	gboolean                    show_avatars;
 	gboolean                    show_groups;
 	gboolean                    is_compact;
+	gboolean                    show_protocols;
 	gboolean                    show_active;
 	EmpathyContactListStoreSort sort_criterium;
 	guint                       inhibit_active;
 	guint                       setup_idle_id;
 	gboolean                    dispose_has_run;
+	GHashTable                  *status_icons;
 } EmpathyContactListStorePriv;
 
 typedef struct {
@@ -103,6 +105,12 @@ static void             contact_list_store_members_changed_cb        (EmpathyCon
 								      gchar                         *message,
 								      gboolean                       is_member,
 								      EmpathyContactListStore       *store);
+static void             contact_list_store_member_renamed_cb         (EmpathyContactList            *list_iface,
+								      EmpathyContact                *old_contact,
+								      EmpathyContact                *new_contact,
+								      guint                          reason,
+								      gchar                         *message,
+								      EmpathyContactListStore       *store);
 static void             contact_list_store_groups_changed_cb         (EmpathyContactList            *list_iface,
 								      EmpathyContact                *contact,
 								      gchar                         *group,
@@ -159,6 +167,7 @@ enum {
 	PROP_CONTACT_LIST,
 	PROP_SHOW_OFFLINE,
 	PROP_SHOW_AVATARS,
+	PROP_SHOW_PROTOCOLS,
 	PROP_SHOW_GROUPS,
 	PROP_IS_COMPACT,
 	PROP_SORT_CRITERIUM
@@ -174,6 +183,10 @@ contact_list_store_iface_setup (gpointer user_data)
 	GList                       *contacts, *l;
 
 	/* Signal connection. */
+	g_signal_connect (priv->list,
+			  "member-renamed",
+			  G_CALLBACK (contact_list_store_member_renamed_cb),
+			  store);
 	g_signal_connect (priv->list,
 			  "members-changed",
 			  G_CALLBACK (contact_list_store_members_changed_cb),
@@ -245,6 +258,14 @@ empathy_contact_list_store_class_init (EmpathyContactListStoreClass *klass)
 								"avatars for contacts",
 								TRUE,
 								G_PARAM_READWRITE));
+	 g_object_class_install_property (object_class,
+					  PROP_SHOW_PROTOCOLS,
+					  g_param_spec_boolean ("show-protocols",
+								"Show Protocols",
+								"Whether contact list should display "
+								"protocols for contacts",
+								FALSE,
+								G_PARAM_READWRITE));
 	 g_object_class_install_property (object_class,
 					  PROP_SHOW_GROUPS,
 					  g_param_spec_boolean ("show-groups",
@@ -282,9 +303,11 @@ empathy_contact_list_store_init (EmpathyContactListStore *store)
 	store->priv = priv;
 	priv->show_avatars = TRUE;
 	priv->show_groups = TRUE;
+	priv->show_protocols = FALSE;
 	priv->inhibit_active = g_timeout_add_seconds (ACTIVE_USER_WAIT_TO_ENABLE_TIME,
 						      (GSourceFunc) contact_list_store_inibit_active_cb,
 						      store);
+	priv->status_icons = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 	contact_list_store_setup (store);
 }
 
@@ -308,6 +331,9 @@ contact_list_store_dispose (GObject *object)
 	}
 	g_list_free (contacts);
 
+	g_signal_handlers_disconnect_by_func (priv->list,
+					      G_CALLBACK (contact_list_store_member_renamed_cb),
+					      object);
 	g_signal_handlers_disconnect_by_func (priv->list,
 					      G_CALLBACK (contact_list_store_members_changed_cb),
 					      object);
@@ -324,6 +350,7 @@ contact_list_store_dispose (GObject *object)
 		g_source_remove (priv->setup_idle_id);
 	}
 
+	g_hash_table_destroy (priv->status_icons);
 	G_OBJECT_CLASS (empathy_contact_list_store_parent_class)->dispose (object);
 }
 
@@ -347,6 +374,9 @@ contact_list_store_get_property (GObject    *object,
 	case PROP_SHOW_AVATARS:
 		g_value_set_boolean (value, priv->show_avatars);
 		break;
+	case PROP_SHOW_PROTOCOLS:
+		g_value_set_boolean (value, priv->show_protocols);
+		break;
 	case PROP_SHOW_GROUPS:
 		g_value_set_boolean (value, priv->show_groups);
 		break;
@@ -385,6 +415,10 @@ contact_list_store_set_property (GObject      *object,
 		empathy_contact_list_store_set_show_avatars (EMPATHY_CONTACT_LIST_STORE (object),
 							    g_value_get_boolean (value));
 		break;
+	case PROP_SHOW_PROTOCOLS:
+		empathy_contact_list_store_set_show_protocols (EMPATHY_CONTACT_LIST_STORE (object),
+							    g_value_get_boolean (value));
+		break;
 	case PROP_SHOW_GROUPS:
 		empathy_contact_list_store_set_show_groups (EMPATHY_CONTACT_LIST_STORE (object),
 							    g_value_get_boolean (value));
@@ -504,6 +538,42 @@ empathy_contact_list_store_set_show_avatars (EmpathyContactListStore *store,
 	g_object_notify (G_OBJECT (store), "show-avatars");
 }
 
+
+gboolean
+empathy_contact_list_store_get_show_protocols (EmpathyContactListStore *store)
+{
+	EmpathyContactListStorePriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_STORE (store), TRUE);
+
+	priv = GET_PRIV (store);
+
+	return priv->show_protocols;
+}
+
+void
+empathy_contact_list_store_set_show_protocols (EmpathyContactListStore *store,
+					    gboolean                show_protocols)
+{
+	EmpathyContactListStorePriv *priv;
+	GtkTreeModel               *model;
+
+	g_return_if_fail (EMPATHY_IS_CONTACT_LIST_STORE (store));
+
+	priv = GET_PRIV (store);
+
+	priv->show_protocols = show_protocols;
+
+	model = GTK_TREE_MODEL (store);
+
+	gtk_tree_model_foreach (model,
+				(GtkTreeModelForeachFunc)
+				contact_list_store_update_list_mode_foreach,
+				store);
+
+	g_object_notify (G_OBJECT (store), "show-protocols");
+}
+
 gboolean
 empathy_contact_list_store_get_show_groups (EmpathyContactListStore *store)
 {
@@ -740,7 +810,7 @@ contact_list_store_setup (EmpathyContactListStore *store)
 {
 	EmpathyContactListStorePriv *priv;
 	GType types[] = {
-		G_TYPE_STRING,        /* Status icon-name */
+		GDK_TYPE_PIXBUF,      /* Status pixbuf */
 		GDK_TYPE_PIXBUF,      /* Avatar pixbuf */
 		G_TYPE_BOOLEAN,       /* Avatar pixbuf visible */
 		G_TYPE_STRING,        /* Name */
@@ -789,6 +859,38 @@ contact_list_store_inibit_active_cb (EmpathyContactListStore *store)
 	return FALSE;
 }
 
+static void
+contact_list_store_add_contact_and_connect (EmpathyContactListStore *store, EmpathyContact *contact)
+{
+	g_signal_connect (contact, "notify::presence",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+	g_signal_connect (contact, "notify::presence-message",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+	g_signal_connect (contact, "notify::name",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+	g_signal_connect (contact, "notify::avatar",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+	g_signal_connect (contact, "notify::capabilities",
+			  G_CALLBACK (contact_list_store_contact_updated_cb),
+			  store);
+
+	contact_list_store_add_contact (store, contact);
+}
+
+static void
+contact_list_store_remove_contact_and_disconnect (EmpathyContactListStore *store, EmpathyContact *contact)
+{
+	g_signal_handlers_disconnect_by_func (contact,
+					      G_CALLBACK (contact_list_store_contact_updated_cb),
+					      store);
+
+	contact_list_store_remove_contact (store, contact);
+}
+
 static void
 contact_list_store_members_changed_cb (EmpathyContactList      *list_iface,
 				       EmpathyContact          *contact,
@@ -808,32 +910,37 @@ contact_list_store_members_changed_cb (EmpathyContactList      *list_iface,
 		is_member ? "added" : "removed");
 
 	if (is_member) {
-		g_signal_connect (contact, "notify::presence",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-		g_signal_connect (contact, "notify::presence-message",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-		g_signal_connect (contact, "notify::name",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-		g_signal_connect (contact, "notify::avatar",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-		g_signal_connect (contact, "notify::capabilities",
-				  G_CALLBACK (contact_list_store_contact_updated_cb),
-				  store);
-
-		contact_list_store_add_contact (store, contact);
+		contact_list_store_add_contact_and_connect (store, contact);
 	} else {
-		g_signal_handlers_disconnect_by_func (contact,
-						      G_CALLBACK (contact_list_store_contact_updated_cb),
-						      store);
-
-		contact_list_store_remove_contact (store, contact);
+		contact_list_store_remove_contact_and_disconnect (store, contact);
 	}
 }
 
+static void
+contact_list_store_member_renamed_cb (EmpathyContactList      *list_iface,
+				      EmpathyContact          *old_contact,
+				      EmpathyContact          *new_contact,
+				      guint                    reason,
+				      gchar                   *message,
+				      EmpathyContactListStore *store)
+{
+	EmpathyContactListStorePriv *priv;
+
+	priv = GET_PRIV (store);
+
+	DEBUG ("Contact %s (%d) renamed to %s (%d)",
+		empathy_contact_get_id (old_contact),
+		empathy_contact_get_handle (old_contact),
+		empathy_contact_get_id (new_contact),
+		empathy_contact_get_handle (new_contact));
+
+	/* add the new contact */
+	contact_list_store_add_contact_and_connect (store, new_contact);
+
+	/* remove old contact */
+	contact_list_store_remove_contact_and_disconnect (store, old_contact);
+}
+
 static void
 contact_list_store_groups_changed_cb (EmpathyContactList      *list_iface,
 				      EmpathyContact          *contact,
@@ -889,7 +996,6 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 	}
 	/* If no groups just add it at the top level. */
 	if (!groups) {
-		GtkTreeIter iter;
 		GtkTreeModel *model = GTK_TREE_MODEL (store);
 
 		if (gtk_tree_model_get_iter_first (model, &iter)) do {
@@ -1006,7 +1112,9 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	gboolean                    do_set_active = FALSE;
 	gboolean                    do_set_refresh = FALSE;
 	gboolean                    show_avatar = FALSE;
+	gboolean                    show_protocol = FALSE;
 	GdkPixbuf                  *pixbuf_avatar;
+	GdkPixbuf                  *pixbuf_status;
 
 	priv = GET_PRIV (store);
 
@@ -1099,10 +1207,14 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	if (priv->show_avatars && !priv->is_compact) {
 		show_avatar = TRUE;
 	}
+	if (priv->show_protocols && !priv->is_compact) {
+		show_protocol = TRUE;
+	}
 	pixbuf_avatar = empathy_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+	pixbuf_status = contact_list_store_get_contact_status_icon (store, contact);
 	for (l = iters; l && set_model; l = l->next) {
 		gtk_tree_store_set (GTK_TREE_STORE (store), l->data,
-				    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, empathy_icon_name_for_contact (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, pixbuf_status,
 				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR, pixbuf_avatar,
 				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
 				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
@@ -1193,19 +1305,19 @@ contact_list_store_contact_set_active (EmpathyContactListStore *store,
 static ShowActiveData *
 contact_list_store_contact_active_new (EmpathyContactListStore *store,
 				       EmpathyContact          *contact,
-				       gboolean                remove)
+				       gboolean                remove_)
 {
 	ShowActiveData *data;
 
 	DEBUG ("Contact:'%s' now active, and %s be removed",
 		empathy_contact_get_name (contact),
-		remove ? "WILL" : "WILL NOT");
+		remove_ ? "WILL" : "WILL NOT");
 
 	data = g_slice_new0 (ShowActiveData);
 
 	data->store = g_object_ref (store);
 	data->contact = g_object_ref (contact);
-	data->remove = remove;
+	data->remove = remove_;
 
 	return data;
 }
@@ -1532,15 +1644,32 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 					     EmpathyContactListStore *store)
 {
 	EmpathyContactListStorePriv *priv;
-	gboolean                    show_avatar = FALSE;
+	gboolean                     show_avatar = FALSE;
+	gboolean                     show_protocol = FALSE;
+	EmpathyContact              *contact;
+	GdkPixbuf                   *pixbuf_status;
 
 	priv = GET_PRIV (store);
 
 	if (priv->show_avatars && !priv->is_compact) {
 		show_avatar = TRUE;
 	}
+	if (priv->show_protocols && !priv->is_compact) {
+		show_protocol = TRUE;
+	}
+
+	gtk_tree_model_get (model, iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
+
+	if (contact == NULL){
+		return FALSE;
+	}
+	/* get icon from hash_table */
+	pixbuf_status = contact_list_store_get_contact_status_icon (store, contact);
 
 	gtk_tree_store_set (GTK_TREE_STORE (store), iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, pixbuf_status,
 			    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
 			    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, !priv->is_compact,
 			    -1);
@@ -1548,3 +1677,59 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 	return FALSE;
 }
 
+static GdkPixbuf *
+contact_list_store_get_contact_status_icon_with_icon_name (
+					EmpathyContactListStore *store,
+					EmpathyContact *contact,
+					const gchar *status_icon_name)
+{
+	GdkPixbuf                   *pixbuf_status = NULL;
+	EmpathyContactListStorePriv *priv;
+	const gchar                 *protocol_name = NULL;
+	gchar                       *icon_name = NULL;
+	gboolean                     show_protocol = FALSE;
+
+	priv = GET_PRIV (store);
+
+	if (priv->show_protocols && !priv->is_compact) {
+		show_protocol = TRUE;
+	}
+	if (show_protocol) {
+		protocol_name = empathy_protocol_name_for_contact (contact);
+		icon_name = g_strdup_printf ("%s-%s", status_icon_name, protocol_name);
+	} else {
+		icon_name = g_strdup_printf ("%s", status_icon_name);
+	}
+	pixbuf_status = g_hash_table_lookup (priv->status_icons, icon_name);
+	if (pixbuf_status == NULL) {
+		pixbuf_status = empathy_pixbuf_contact_status_icon_with_icon_name (contact,
+				    status_icon_name,
+				    show_protocol);
+		if (pixbuf_status != NULL) {
+			g_hash_table_insert (priv->status_icons,
+			    g_strdup (icon_name),
+			    pixbuf_status);
+			DEBUG( "Created status icon %s\n", icon_name);
+		}
+	} else {
+		DEBUG( "retrieved from cache status icon %s\n", icon_name);
+	}
+	g_free (icon_name);
+	return pixbuf_status;
+}
+
+GdkPixbuf *
+contact_list_store_get_contact_status_icon (EmpathyContactListStore *store,
+					    EmpathyContact *contact)
+{
+	GdkPixbuf                   *pixbuf_status = NULL;
+	const gchar                 *status_icon_name = NULL;
+
+	status_icon_name = empathy_icon_name_for_contact (contact);
+	pixbuf_status = contact_list_store_get_contact_status_icon_with_icon_name (
+			    store,
+			    contact,
+			    status_icon_name);
+
+	return pixbuf_status;
+}
