@@ -54,6 +54,15 @@ contact_list_base_init (gpointer klass)
 	static gboolean initialized = FALSE;
 
 	if (!initialized) {
+		g_signal_new ("member-renamed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      _empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING,
+			      G_TYPE_NONE,
+			      4, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT, G_TYPE_UINT, G_TYPE_STRING);
+
 		g_signal_new ("members-changed",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -125,6 +134,18 @@ empathy_contact_list_get_members (EmpathyContactList *list)
 	return NULL;
 }
 
+EmpathyContactMonitor *
+empathy_contact_list_get_monitor (EmpathyContactList *list)
+{
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST (list), NULL);
+
+	if (EMPATHY_CONTACT_LIST_GET_IFACE (list)->get_monitor) {
+		return EMPATHY_CONTACT_LIST_GET_IFACE (list)->get_monitor (list);
+	}
+
+	return NULL;
+}
+
 GList *
 empathy_contact_list_get_pendings (EmpathyContactList *list)
 {
@@ -217,3 +238,14 @@ empathy_contact_list_remove_group (EmpathyContactList *list,
 	}
 }
 
+EmpathyContactListFlags
+empathy_contact_list_get_flags (EmpathyContactList *list)
+{
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST (list), 0);
+
+	if (EMPATHY_CONTACT_LIST_GET_IFACE (list)->get_flags) {
+		return EMPATHY_CONTACT_LIST_GET_IFACE (list)->get_flags (list);
+	} else {
+		return 0;
+	}
+}
