@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- * 
+ *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
 
@@ -23,10 +23,10 @@
 
 #include <string.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/enums.h>
 
 #include "empathy-contact-manager.h"
-#include "empathy-account-manager.h"
 #include "empathy-contact-monitor.h"
 #include "empathy-contact-list.h"
 #include "empathy-utils.h"
@@ -37,7 +37,7 @@
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyContactManager)
 typedef struct {
 	GHashTable     *lists;
-	EmpathyAccountManager *account_manager;
+	TpAccountManager *account_manager;
 	EmpathyContactMonitor *contact_monitor;
 } EmpathyContactManagerPriv;
 
@@ -86,14 +86,33 @@ contact_manager_groups_changed_cb (EmpathyTpContactList  *list,
 			       contact, group, is_member);
 }
 
-static void contact_manager_destroy_cb (EmpathyTpContactList  *list,
-					EmpathyContactManager *manager);
+static void
+contact_manager_invalidated_cb (TpProxy *connection,
+				guint    domain,
+				gint     code,
+				gchar   *message,
+				EmpathyContactManager *manager)
+{
+	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
+	EmpathyTpContactList *list;
+
+	DEBUG ("Removing connection: %s (%s)",
+		tp_proxy_get_object_path (TP_PROXY (connection)),
+		message);
+
+	list = g_hash_table_lookup (priv->lists, connection);
+	if (list) {
+		empathy_tp_contact_list_remove_all (list);
+		g_hash_table_remove (priv->lists, connection);
+	}
+}
 
 static void
 contact_manager_disconnect_foreach (gpointer key,
 				    gpointer value,
 				    gpointer user_data)
 {
+	TpConnection *connection = key;
 	EmpathyTpContactList  *list = value;
 	EmpathyContactManager *manager = user_data;
 
@@ -107,75 +126,53 @@ contact_manager_disconnect_foreach (gpointer key,
 	g_signal_handlers_disconnect_by_func (list,
 					      contact_manager_groups_changed_cb,
 					      manager);
-	g_signal_handlers_disconnect_by_func (list,
-					      contact_manager_destroy_cb,
+	g_signal_handlers_disconnect_by_func (connection,
+					      contact_manager_invalidated_cb,
 					      manager);
 }
 
 static void
-contact_manager_destroy_cb (EmpathyTpContactList  *list,
-			    EmpathyContactManager *manager)
+contact_manager_status_changed_cb (TpAccount *account,
+				   guint old_status,
+				   guint new_status,
+				   guint reason,
+				   gchar *dbus_error_name,
+				   GHashTable *details,
+				   EmpathyContactManager *self)
 {
-	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
-	McAccount                 *account;
-
-	account = empathy_tp_contact_list_get_account (list);
-
-	DEBUG ("Removing account: %s", mc_account_get_display_name (account));
-
-	contact_manager_disconnect_foreach (account, list, manager);
-	g_hash_table_remove (priv->lists, account);
-}
-
-static void
-contact_manager_add_account (EmpathyContactManager *manager,
-			     McAccount             *account)
-{
-	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
+	EmpathyContactManagerPriv *priv = GET_PRIV (self);
 	EmpathyTpContactList      *list;
+	TpConnection              *connection;
 
-	if (g_hash_table_lookup (priv->lists, account)) {
+	if (new_status == TP_CONNECTION_STATUS_DISCONNECTED)
+		/* No point to start tracking a connection which is about to die */
+		return;
+
+	connection = tp_account_get_connection (account);
+
+	if (connection == NULL || g_hash_table_lookup (priv->lists, connection)) {
 		return;
 	}
 
-	DEBUG ("Adding new account: %s", mc_account_get_display_name (account));
+	DEBUG ("Adding new connection: %s",
+		tp_proxy_get_object_path (TP_PROXY (connection)));
 
-	list = empathy_tp_contact_list_new (account);
-	if (!list) {
-		return;
-	}
-
-	g_hash_table_insert (priv->lists, g_object_ref (account), list);
+	list = empathy_tp_contact_list_new (connection);
+	g_hash_table_insert (priv->lists, g_object_ref (connection), list);
+	g_signal_connect (connection, "invalidated",
+			  G_CALLBACK (contact_manager_invalidated_cb),
+			  self);
 
 	/* Connect signals */
 	g_signal_connect (list, "members-changed",
 			  G_CALLBACK (contact_manager_members_changed_cb),
-			  manager);
+			  self);
 	g_signal_connect (list, "pendings-changed",
 			  G_CALLBACK (contact_manager_pendings_changed_cb),
-			  manager);
+			  self);
 	g_signal_connect (list, "groups-changed",
 			  G_CALLBACK (contact_manager_groups_changed_cb),
-			  manager);
-	g_signal_connect (list, "destroy",
-			  G_CALLBACK (contact_manager_destroy_cb),
-			  manager);
-}
-
-static void
-contact_manager_connection_changed_cb (EmpathyAccountManager *account_manager,
-				       McAccount *account,
-				       TpConnectionStatusReason  reason,
-				       TpConnectionStatus current,
-				       TpConnectionStatus previous,
-				       EmpathyContactManager *manager)
-{
-	if (current != TP_CONNECTION_STATUS_CONNECTED) {
-		/* We only care about newly connected accounts */
-		return;
-	}
-
-	contact_manager_add_account (manager, account);
+			  self);
 }
 
 static void
@@ -188,9 +185,6 @@ contact_manager_finalize (GObject *object)
 			      object);
 	g_hash_table_destroy (priv->lists);
 
-	g_signal_handlers_disconnect_by_func (priv->account_manager,
-					      contact_manager_connection_changed_cb,
-					      object);
 	g_object_unref (priv->account_manager);
 
 	if (priv->contact_monitor) {
@@ -212,12 +206,29 @@ contact_manager_constructor (GType type,
 			(type, n_props, props);
 
 		manager_singleton = EMPATHY_CONTACT_MANAGER (retval);
-		g_object_add_weak_pointer (retval, (gpointer *) &manager_singleton);
+		g_object_add_weak_pointer (retval, (gpointer) &manager_singleton);
 	}
 
 	return retval;
 }
 
+/**
+ * empathy_contact_manager_initialized:
+ *
+ * Reports whether or not the singleton has already been created.
+ *
+ * There can be instances where you want to access the #EmpathyContactManager
+ * only if it has been set up for this process.
+ *
+ * Returns: %TRUE if the #EmpathyContactManager singleton has previously
+ * been initialized.
+ */
+gboolean
+empathy_contact_manager_initialized (void)
+{
+	return (manager_singleton != NULL);
+}
+
 static void
 empathy_contact_manager_class_init (EmpathyContactManagerClass *klass)
 {
@@ -229,37 +240,56 @@ empathy_contact_manager_class_init (EmpathyContactManagerClass *klass)
 	g_type_class_add_private (object_class, sizeof (EmpathyContactManagerPriv));
 }
 
+static void
+account_manager_prepared_cb (GObject *source_object,
+			     GAsyncResult *result,
+			     gpointer user_data)
+{
+	GList *accounts, *l;
+	EmpathyContactManager *manager = user_data;
+	TpAccountManager *account_manager = TP_ACCOUNT_MANAGER (source_object);
+	GError *error = NULL;
+
+	if (!tp_account_manager_prepare_finish (account_manager, result, &error)) {
+		DEBUG ("Failed to prepare account manager: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	accounts = tp_account_manager_get_valid_accounts (account_manager);
+
+	for (l = accounts; l != NULL; l = l->next) {
+		TpAccount *account = l->data;
+		TpConnection *conn = tp_account_get_connection (account);
+
+		if (conn != NULL) {
+			contact_manager_status_changed_cb (account, 0, 0, 0,
+							   NULL, NULL, manager);
+		}
+
+		empathy_signal_connect_weak (account, "status-changed",
+		    G_CALLBACK (contact_manager_status_changed_cb),
+		    G_OBJECT (manager));
+	}
+	g_list_free (accounts);
+}
+
 static void
 empathy_contact_manager_init (EmpathyContactManager *manager)
 {
-	GSList                    *accounts, *l;
-	MissionControl            *mc;
 	EmpathyContactManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
 		EMPATHY_TYPE_CONTACT_MANAGER, EmpathyContactManagerPriv);
 
 	manager->priv = priv;
-	priv->lists = g_hash_table_new_full (empathy_account_hash,
-					     empathy_account_equal,
+	priv->lists = g_hash_table_new_full (empathy_proxy_hash,
+					     empathy_proxy_equal,
 					     (GDestroyNotify) g_object_unref,
 					     (GDestroyNotify) g_object_unref);
-	priv->account_manager = empathy_account_manager_dup_singleton ();
+	priv->account_manager = tp_account_manager_dup ();
 	priv->contact_monitor = NULL;
 
-	g_signal_connect (priv->account_manager,
-			  "account-connection-changed",
-			  G_CALLBACK (contact_manager_connection_changed_cb), manager);
-
-	mc = empathy_mission_control_dup_singleton ();
-
-	/* Get ContactList for existing connections */
-	accounts = mission_control_get_online_connections (mc, NULL);
-	for (l = accounts; l; l = l->next) {
-		contact_manager_add_account (manager, l->data);
-		g_object_unref (l->data);
-	}
-
-	g_slist_free (accounts);
-	g_object_unref (mc);
+	tp_account_manager_prepare_async (priv->account_manager, NULL,
+	    account_manager_prepared_cb, manager);
 }
 
 EmpathyContactManager *
@@ -270,14 +300,14 @@ empathy_contact_manager_dup_singleton (void)
 
 EmpathyTpContactList *
 empathy_contact_manager_get_list (EmpathyContactManager *manager,
-				  McAccount             *account)
+				  TpConnection          *connection)
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager), NULL);
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
+	g_return_val_if_fail (TP_IS_CONNECTION (connection), NULL);
 
-	return g_hash_table_lookup (priv->lists, account);
+	return g_hash_table_lookup (priv->lists, connection);
 }
 
 static void
@@ -287,12 +317,12 @@ contact_manager_add (EmpathyContactList *manager,
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
 	EmpathyContactList        *list;
-	McAccount                 *account;
+	TpConnection              *connection;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager));
 
-	account = empathy_contact_get_account (contact);
-	list = g_hash_table_lookup (priv->lists, account);
+	connection = empathy_contact_get_connection (contact);
+	list = g_hash_table_lookup (priv->lists, connection);
 
 	if (list) {
 		empathy_contact_list_add (list, contact, message);
@@ -301,17 +331,17 @@ contact_manager_add (EmpathyContactList *manager,
 
 static void
 contact_manager_remove (EmpathyContactList *manager,
-			EmpathyContact      *contact,
+			EmpathyContact     *contact,
 			const gchar        *message)
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
 	EmpathyContactList        *list;
-	McAccount                 *account;
+	TpConnection              *connection;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager));
 
-	account = empathy_contact_get_account (contact);
-	list = g_hash_table_lookup (priv->lists, account);
+	connection = empathy_contact_get_connection (contact);
+	list = g_hash_table_lookup (priv->lists, connection);
 
 	if (list) {
 		empathy_contact_list_remove (list, contact, message);
@@ -319,7 +349,7 @@ contact_manager_remove (EmpathyContactList *manager,
 }
 
 static void
-contact_manager_get_members_foreach (McAccount             *account,
+contact_manager_get_members_foreach (TpConnection          *connection,
 				     EmpathyTpContactList  *list,
 				     GList                **contacts)
 {
@@ -357,7 +387,7 @@ contact_manager_get_monitor (EmpathyContactList *manager)
 }
 
 static void
-contact_manager_get_pendings_foreach (McAccount             *account,
+contact_manager_get_pendings_foreach (TpConnection          *connection,
 				      EmpathyTpContactList  *list,
 				      GList                **contacts)
 {
@@ -383,7 +413,7 @@ contact_manager_get_pendings (EmpathyContactList *manager)
 }
 
 static void
-contact_manager_get_all_groups_foreach (McAccount             *account,
+contact_manager_get_all_groups_foreach (TpConnection          *connection,
 					EmpathyTpContactList  *list,
 					GList                **all_groups)
 {
@@ -424,12 +454,12 @@ contact_manager_get_groups (EmpathyContactList *manager,
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
 	EmpathyContactList        *list;
-	McAccount                 *account;
+	TpConnection              *connection;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager), NULL);
 
-	account = empathy_contact_get_account (contact);
-	list = g_hash_table_lookup (priv->lists, account);
+	connection = empathy_contact_get_connection (contact);
+	list = g_hash_table_lookup (priv->lists, connection);
 
 	if (list) {
 		return empathy_contact_list_get_groups (list, contact);
@@ -445,12 +475,12 @@ contact_manager_add_to_group (EmpathyContactList *manager,
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
 	EmpathyContactList        *list;
-	McAccount                 *account;
+	TpConnection              *connection;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager));
 
-	account = empathy_contact_get_account (contact);
-	list = g_hash_table_lookup (priv->lists, account);
+	connection = empathy_contact_get_connection (contact);
+	list = g_hash_table_lookup (priv->lists, connection);
 
 	if (list) {
 		empathy_contact_list_add_to_group (list, contact, group);
@@ -464,12 +494,12 @@ contact_manager_remove_from_group (EmpathyContactList *manager,
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
 	EmpathyContactList        *list;
-	McAccount                 *account;
+	TpConnection              *connection;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager));
 
-	account = empathy_contact_get_account (contact);
-	list = g_hash_table_lookup (priv->lists, account);
+	connection = empathy_contact_get_connection (contact);
+	list = g_hash_table_lookup (priv->lists, connection);
 
 	if (list) {
 		empathy_contact_list_remove_from_group (list, contact, group);
@@ -482,7 +512,7 @@ typedef struct {
 } RenameGroupData;
 
 static void
-contact_manager_rename_group_foreach (McAccount            *account,
+contact_manager_rename_group_foreach (TpConnection         *connection,
 				      EmpathyTpContactList *list,
 				      RenameGroupData      *data)
 {
@@ -508,7 +538,7 @@ contact_manager_rename_group (EmpathyContactList *manager,
 			      &data);
 }
 
-static void contact_manager_remove_group_foreach (McAccount	*account,
+static void contact_manager_remove_group_foreach (TpConnection         *connection,
 						  EmpathyTpContactList *list,
 						  const gchar *group)
 {
@@ -521,7 +551,7 @@ contact_manager_remove_group (EmpathyContactList *manager,
 			      const gchar *group)
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
-	
+
 	g_return_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager));
 
 	g_hash_table_foreach (priv->lists,
@@ -545,19 +575,24 @@ contact_manager_iface_init (EmpathyContactListIface *iface)
 	iface->remove_group	 = contact_manager_remove_group;
 }
 
-gboolean
-empathy_contact_manager_can_add (EmpathyContactManager *manager,
-				 McAccount             *account)
+EmpathyContactListFlags
+empathy_contact_manager_get_flags_for_connection (
+				EmpathyContactManager *manager,
+				TpConnection          *connection)
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
-	EmpathyTpContactList      *list;
-	
+	EmpathyContactList        *list;
+	EmpathyContactListFlags    flags;
+
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager), FALSE);
+	g_return_val_if_fail (connection != NULL, FALSE);
 
-	list = g_hash_table_lookup (priv->lists, account);
-	if (list == NULL)
+	list = g_hash_table_lookup (priv->lists, connection);
+	if (list == NULL) {
 		return FALSE;
+	}
+	flags = empathy_contact_list_get_flags (list);
 
-	return empathy_tp_contact_list_can_add (list);
+	return flags;
 }
 
