@@ -37,6 +37,7 @@
 
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-connection-managers.h>
+#include <libempathy/empathy-connectivity.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 
 #include <libempathy-gtk/empathy-protocol-chooser.h>
@@ -50,6 +51,7 @@
 #include "empathy-accounts-dialog.h"
 #include "empathy-import-dialog.h"
 #include "empathy-import-utils.h"
+#include "ephy-spinner.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_ACCOUNT
 #include <libempathy/empathy-debug.h>
@@ -75,8 +77,13 @@ typedef struct {
   GtkWidget *window;
 
   GtkWidget *alignment_settings;
+  GtkWidget *alignment_infobar;
 
   GtkWidget *vbox_details;
+  GtkWidget *infobar;
+  GtkWidget *label_status;
+  GtkWidget *image_status;
+  GtkWidget *throbber;
   GtkWidget *frame_no_protocol;
 
   GtkWidget *treeview;
@@ -166,6 +173,145 @@ accounts_dialog_update_name_label (EmpathyAccountsDialog *dialog,
   g_free (text);
 }
 
+static void
+accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
+    TpAccount *account)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  const gchar               *message;
+  gchar                     *status_message = NULL;
+  guint                     status;
+  guint                     reason;
+  guint                     presence;
+  EmpathyConnectivity       *connectivity;
+  GtkTreeView               *view;
+  GtkTreeModel              *model;
+  GtkTreeSelection          *selection;
+  GtkTreeIter               iter;
+  TpAccount                 *selected_account;
+  gboolean                  account_enabled;
+  gboolean                  creating_account;
+
+  view = GTK_TREE_VIEW (priv->treeview);
+  selection = gtk_tree_view_get_selection (view);
+
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+    return;
+
+  gtk_tree_model_get (model, &iter,
+      COL_ACCOUNT_POINTER, &selected_account, -1);
+
+  /* do not update the infobar when the account is not selected */
+  if (account != selected_account)
+    return;
+
+  if (account != NULL)
+    {
+      status = tp_account_get_connection_status (account, &reason);
+      presence = tp_account_get_current_presence (account, NULL, &status_message);
+      account_enabled = tp_account_is_enabled (account);
+      creating_account = FALSE;
+    }
+  else
+    {
+      status = TP_CONNECTION_STATUS_DISCONNECTED;
+      presence = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
+      account_enabled = FALSE;
+      creating_account = TRUE;
+    }
+
+  gtk_image_set_from_icon_name (GTK_IMAGE (priv->image_status),
+      empathy_icon_name_for_presence (presence), GTK_ICON_SIZE_SMALL_TOOLBAR);
+
+  if (account_enabled)
+    {
+      switch (status)
+        {
+          case TP_CONNECTION_STATUS_CONNECTING:
+            message = _("Connecting...");
+            gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                GTK_MESSAGE_INFO);
+
+            ephy_spinner_start (EPHY_SPINNER (priv->throbber));
+            gtk_widget_show (priv->throbber);
+            gtk_widget_hide (priv->image_status);
+            break;
+          case TP_CONNECTION_STATUS_CONNECTED:
+            if (g_strcmp0 (status_message, "") == 0)
+              {
+                message = g_strdup_printf ("%s",
+                    empathy_presence_get_default_message (presence));
+              }
+            else
+              {
+                message = g_strdup_printf ("%s - %s",
+                    empathy_presence_get_default_message (presence),
+                    status_message);
+              }
+            gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                GTK_MESSAGE_INFO);
+
+            gtk_widget_show (priv->image_status);
+            gtk_widget_hide (priv->throbber);
+            break;
+          case TP_CONNECTION_STATUS_DISCONNECTED:
+            message = g_strdup_printf (_("Disconnected - %s"),
+                empathy_status_reason_get_default_message (reason));
+
+            if (reason == TP_CONNECTION_STATUS_REASON_REQUESTED)
+              {
+                message = g_strdup_printf (_("Offline - %s"),
+                    empathy_status_reason_get_default_message (reason));
+                gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                    GTK_MESSAGE_WARNING);
+              }
+            else
+              {
+                gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                    GTK_MESSAGE_ERROR);
+              }
+
+            connectivity = empathy_connectivity_dup_singleton ();
+            if (!empathy_connectivity_is_online (connectivity))
+               message = _("Offline - No Network Connection");
+
+            g_object_unref (connectivity);
+            ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+            gtk_widget_show (priv->image_status);
+            gtk_widget_hide (priv->throbber);
+            break;
+          default:
+            message = _("Unknown Status");
+            gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                GTK_MESSAGE_WARNING);
+
+            ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+            gtk_widget_hide (priv->image_status);
+            gtk_widget_hide (priv->throbber);
+        }
+    }
+  else
+    {
+      message = _("Offline - Account disabled");
+
+      gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+          GTK_MESSAGE_WARNING);
+      ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+      gtk_widget_show (priv->image_status);
+      gtk_widget_hide (priv->throbber);
+    }
+
+  gtk_label_set_text (GTK_LABEL (priv->label_status), message);
+  gtk_widget_show (priv->label_status);
+
+  if (!creating_account)
+    gtk_widget_show (priv->infobar);
+  else
+    gtk_widget_hide (priv->infobar);
+
+  g_free (status_message);
+}
+
 static void
 empathy_account_dialog_widget_cancelled_cb (
     EmpathyAccountWidget *widget_object,
@@ -225,6 +371,8 @@ empathy_account_dialog_account_created_cb (EmpathyAccountWidget *widget_object,
   g_free (display_name);
 
   accounts_dialog_update_settings (dialog, settings);
+  accounts_dialog_update_status_infobar (dialog,
+      empathy_account_settings_get_account (settings));
 
   if (settings)
     g_object_unref (settings);
@@ -235,7 +383,8 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
     EmpathyAccountSettings *settings)
 {
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
-  gchar *icon_name;
+  gchar                     *icon_name;
+  TpAccount                 *account;
 
   priv->setting_widget_object =
       empathy_account_widget_new_for_protocol (settings, FALSE);
@@ -243,8 +392,6 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
   priv->settings_widget =
       empathy_account_widget_get_widget (priv->setting_widget_object);
 
-  priv->settings_widget =
-      empathy_account_widget_get_widget (priv->setting_widget_object);
   g_signal_connect (priv->setting_widget_object, "account-created",
         G_CALLBACK (empathy_account_dialog_account_created_cb), dialog);
   g_signal_connect (priv->setting_widget_object, "cancelled",
@@ -272,6 +419,9 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
 
   accounts_dialog_update_name_label (dialog,
       empathy_account_settings_get_display_name (settings));
+
+  account = empathy_account_settings_get_account (settings);
+  accounts_dialog_update_status_infobar (dialog, account);
 }
 
 static void
@@ -1128,6 +1278,9 @@ accounts_dialog_connection_changed_cb (TpAccount *account,
   gboolean      found;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
+  /* Update the status-infobar in the details view */
+  accounts_dialog_update_status_infobar (dialog, account);
+
   /* Update the status in the model */
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
@@ -1158,6 +1311,17 @@ accounts_dialog_connection_changed_cb (TpAccount *account,
         dialog);
 }
 
+static void
+accounts_dialog_presence_changed_cb (TpAccount *account,
+    guint presence,
+    gchar *status,
+    gchar *status_message,
+    EmpathyAccountsDialog *dialog)
+{
+  /* Update the status-infobar in the details view */
+  accounts_dialog_update_status_infobar (dialog, account);
+}
+
 static void
 accounts_dialog_account_display_name_changed_cb (TpAccount *account,
   GParamSpec *pspec,
@@ -1290,6 +1454,10 @@ enable_or_disable_account (EmpathyAccountsDialog *dialog,
   /* Update the status in the model */
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
+  /* Update the status-infobar in the details view when disabling */
+  if (!enabled)
+    accounts_dialog_update_status_infobar (dialog, account);
+
   DEBUG ("Account %s is now %s",
       tp_account_get_display_name (account),
       enabled ? "enabled" : "disabled");
@@ -1515,6 +1683,8 @@ accounts_dialog_accounts_setup (EmpathyAccountsDialog *dialog)
 
       empathy_signal_connect_weak (l->data, "status-changed",
           G_CALLBACK (accounts_dialog_connection_changed_cb), G_OBJECT (dialog));
+      empathy_signal_connect_weak (l->data, "presence-changed",
+          G_CALLBACK (accounts_dialog_presence_changed_cb), G_OBJECT (dialog));
     }
   g_list_free (accounts);
 
@@ -1550,6 +1720,7 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
   GtkBuilder                   *gui;
   gchar                        *filename;
   EmpathyAccountsDialogPriv    *priv = GET_PRIV (dialog);
+  GtkWidget                    *content_area;
 #ifdef HAVE_MOBLIN
   GtkWidget                    *action_area;
 #endif
@@ -1561,6 +1732,7 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
       "vbox_details", &priv->vbox_details,
       "frame_no_protocol", &priv->frame_no_protocol,
       "alignment_settings", &priv->alignment_settings,
+      "alignment_infobar", &priv->alignment_infobar,
       "treeview", &priv->treeview,
       "frame_new_account", &priv->frame_new_account,
       "hbox_type", &priv->hbox_type,
@@ -1609,6 +1781,28 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
   if (priv->parent_window)
     gtk_window_set_transient_for (GTK_WINDOW (priv->window),
         priv->parent_window);
+
+  /* set up spinner */
+  priv->throbber = ephy_spinner_new ();
+  ephy_spinner_set_size (EPHY_SPINNER (priv->throbber), GTK_ICON_SIZE_SMALL_TOOLBAR);
+
+  priv->infobar = gtk_info_bar_new ();
+  gtk_container_add (GTK_CONTAINER (priv->alignment_infobar),
+      priv->infobar);
+  gtk_widget_show (priv->infobar);
+
+  priv->image_status = gtk_image_new_from_icon_name (
+            empathy_icon_name_for_presence (
+            TP_CONNECTION_PRESENCE_TYPE_OFFLINE), GTK_ICON_SIZE_SMALL_TOOLBAR);
+
+  priv->label_status = gtk_label_new (NULL);
+
+  content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->infobar));
+  gtk_box_pack_start (GTK_BOX (content_area), priv->throbber,
+      FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (content_area), priv->image_status,
+      FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (content_area), priv->label_status);
 }
 
 static void
