@@ -80,6 +80,9 @@ typedef struct {
 	NotifyNotification *notification;
 	NotificationData *notification_data;
 
+	GtkTargetList *contact_targets;
+	GtkTargetList *file_targets;
+
 	/* Menu items. */
 	GtkUIManager *ui_manager;
 	GtkAction   *menu_conv_insert_smiley;
@@ -106,12 +109,22 @@ static const guint tab_accel_keys[] = {
 
 typedef enum {
 	DND_DRAG_TYPE_CONTACT_ID,
+	DND_DRAG_TYPE_URI_LIST,
 	DND_DRAG_TYPE_TAB
 } DndDragType;
 
 static const GtkTargetEntry drag_types_dest[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 	{ "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, DND_DRAG_TYPE_TAB },
+	{ "text/uri-list", 0, DND_DRAG_TYPE_URI_LIST },
+};
+
+static const GtkTargetEntry drag_types_dest_contact[] = {
+	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
+};
+
+static const GtkTargetEntry drag_types_dest_file[] = {
+	{ "text/uri-list", 0, DND_DRAG_TYPE_URI_LIST },
 };
 
 static void chat_window_update (EmpathyChatWindow *window);
@@ -268,8 +281,8 @@ chat_window_create_label (EmpathyChatWindow *window,
 		/* We don't want focus/keynav for the button to avoid clutter, and
 		 * Ctrl-W works anyway.
 		 */
-		GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_FOCUS);
-		GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_DEFAULT);
+		gtk_widget_set_can_focus (close_button, FALSE);
+		gtk_widget_set_can_default (close_button, FALSE);
 
 		/* Set the name to make the special rc style match. */
 		gtk_widget_set_name (close_button, "empathy-close-button");
@@ -356,16 +369,97 @@ chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
 	}
 }
 
+static guint
+get_all_unread_messages (EmpathyChatWindowPriv *priv)
+{
+	GList *l;
+	guint nb = 0;
+
+	for (l = priv->chats_new_msg; l != NULL; l = g_list_next (l)) {
+		EmpathyChat *chat = l->data;
+
+		nb += empathy_chat_get_nb_unread_messages (chat);
+	}
+
+	return nb;
+}
+
+static gchar *
+get_window_title_name (EmpathyChatWindowPriv *priv)
+{
+	const gchar *active_name;
+	guint nb_chats;
+	guint current_unread_msgs;
+
+	nb_chats = g_list_length (priv->chats);
+	g_assert (nb_chats > 0);
+
+	active_name = empathy_chat_get_name (priv->current_chat);
+
+	current_unread_msgs = empathy_chat_get_nb_unread_messages (
+			priv->current_chat);
+
+	if (nb_chats == 1) {
+		/* only one tab */
+		if (current_unread_msgs == 0)
+			return g_strdup (active_name);
+		else
+			return g_strdup_printf (ngettext (
+				"%s (%d unread)",
+				"%s (%d unread)", current_unread_msgs),
+				active_name, current_unread_msgs);
+	} else {
+		guint nb_others = nb_chats - 1;
+		guint all_unread_msgs;
+
+		all_unread_msgs = get_all_unread_messages (priv);
+
+		if (all_unread_msgs == 0) {
+			/* no unread message */
+			return g_strdup_printf (ngettext (
+				"%s (and %u other)",
+				"%s (and %u others)", nb_others),
+				active_name, nb_others);
+		}
+
+		else if (all_unread_msgs == current_unread_msgs) {
+			/* unread messages are in the current tab */
+			return g_strdup_printf (ngettext (
+				"%s (%d unread)",
+				"%s (%d unread)", current_unread_msgs),
+				active_name, current_unread_msgs);
+		}
+
+		else if (current_unread_msgs == 0) {
+			/* unread messages are in other tabs */
+			return g_strdup_printf (ngettext (
+				"%s (%d unread from others)",
+				"%s (%d unread from others)",
+				all_unread_msgs),
+				active_name, all_unread_msgs);
+		}
+
+		else {
+			/* unread messages are in all the tabs */
+			return g_strdup_printf (ngettext (
+				"%s (%d unread from all)",
+				"%s (%d unread from all)",
+				all_unread_msgs),
+				active_name, all_unread_msgs);
+		}
+	}
+}
+
 static void
 chat_window_title_update (EmpathyChatWindowPriv *priv)
 {
-	const gchar *name;
-
-	name = empathy_chat_get_name (priv->current_chat);
+	gchar *name;
 
 	DEBUG ("Update window : Title");
 
+	name = get_window_title_name (priv);
 	gtk_window_set_title (GTK_WINDOW (priv->dialog), name);
+	g_free (name);
 }
 
 static void
@@ -603,8 +697,8 @@ chat_window_chat_notify_cb (EmpathyChat *chat)
 							      chat);
 		}
 
-		g_object_set_data (G_OBJECT (chat), "chat-window-remote-contact",
-				   remote_contact);
+		g_object_set_data_full (G_OBJECT (chat), "chat-window-remote-contact",
+				   g_object_ref (remote_contact), (GDestroyNotify) g_object_unref);
 	}
 
 	chat_window_update_chat_tab (chat);
@@ -1114,6 +1208,8 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	}
 
 	if (has_focus && priv->current_chat == chat) {
+		/* window and tab are focused so consider the message to be read */
+		empathy_chat_messages_read (chat);
 		return;
 	}
 
@@ -1144,6 +1240,9 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 		    EMPATHY_SOUND_MESSAGE_INCOMING);
 		chat_window_show_or_update_notification (window, message, chat);
 	}
+
+	/* update the number of unread messages */
+	chat_window_title_update (priv);
 }
 
 static GtkNotebook *
@@ -1200,6 +1299,7 @@ chat_window_page_switched_cb (GtkNotebook      *notebook,
 
 	priv->current_chat = chat;
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
+	empathy_chat_messages_read (chat);
 
 	chat_window_update_chat_tab (chat);
 }
@@ -1291,6 +1391,7 @@ chat_window_page_removed_cb (GtkNotebook      *notebook,
 	/* Keep list of chats up to date */
 	priv->chats = g_list_remove (priv->chats, chat);
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
+	empathy_chat_messages_read (chat);
 	priv->chats_composing = g_list_remove (priv->chats_composing, chat);
 
 	if (priv->chats == NULL) {
@@ -1312,6 +1413,7 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	priv = GET_PRIV (window);
 
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, priv->current_chat);
+	empathy_chat_messages_read (priv->current_chat);
 
 	chat_window_set_urgency_hint (window, FALSE);
 
@@ -1321,6 +1423,68 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	return FALSE;
 }
 
+static gboolean
+chat_window_drag_motion (GtkWidget        *widget,
+			 GdkDragContext   *context,
+			 int               x,
+			 int               y,
+			 guint             time_,
+			 EmpathyChatWindow *window)
+{
+	GdkAtom target;
+	EmpathyChatWindowPriv *priv;
+	GdkAtom dest_target;
+
+	priv = GET_PRIV (window);
+
+	target = gtk_drag_dest_find_target (widget, context, NULL);
+
+	dest_target = gdk_atom_intern_static_string ("text/uri-list");
+	if (target == dest_target) {
+		/* This is a file drag.  Ensure the contact is online and set the
+		   drag type to COPY.  Note that it's possible that the tab will
+		   be switched by GTK+ after a timeout from drag_motion without
+		   getting another drag_motion to disable the drop.  You have
+		   to hold your mouse really still.
+		 */
+		EmpathyContact *contact;
+
+		priv = GET_PRIV (window);
+		contact = empathy_chat_get_remote_contact (priv->current_chat);
+		/* contact is NULL for multi-user chats.  We don't do
+		 * file transfers to MUCs.  We also don't send files
+		 * to offline contacts or contacts that don't support
+		 * file transfer.
+		 */
+		if ((contact == NULL) || !empathy_contact_is_online (contact)) {
+			gdk_drag_status (context, 0, time_);
+			return FALSE;
+		}
+		if (!(empathy_contact_get_capabilities (contact)
+			   & EMPATHY_CAPABILITIES_FT)) {
+			gdk_drag_status (context, 0, time_);
+			return FALSE;
+		}
+		gdk_drag_status (context, GDK_ACTION_COPY, time_);
+		return TRUE;
+	}
+
+	dest_target = gdk_atom_intern_static_string ("text/contact-id");
+	if (target == dest_target) {
+		/* This is a drag of a contact from a contact list.  Set to COPY.
+		   FIXME: If this drag is to a MUC window, it invites the user.
+		   Otherwise, it opens a chat.  Should we use a different drag
+		   type for invites?  Should we allow ASK?
+		 */
+		gdk_drag_status (context, GDK_ACTION_COPY, time_);
+		return TRUE;
+	}
+
+	/* Otherwise, it must be a notebook tab drag.  Set to MOVE. */
+	gdk_drag_status (context, GDK_ACTION_MOVE, time_);
+	return TRUE;
+}
+
 static void
 chat_window_drag_data_received (GtkWidget        *widget,
 				GdkDragContext   *context,
@@ -1403,6 +1567,27 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		 */
 		gtk_drag_finish (context, TRUE, FALSE, time_);
 	}
+	else if (info == DND_DRAG_TYPE_URI_LIST) {
+		EmpathyChatWindowPriv *priv;
+		EmpathyContact *contact;
+		const gchar *data;
+
+		priv = GET_PRIV (window);
+		contact = empathy_chat_get_remote_contact (priv->current_chat);
+
+		/* contact is NULL when current_chat is a multi-user chat.
+		 * We don't do file transfers to MUCs, so just cancel the drag.
+		 */
+		if (contact == NULL) {
+			gtk_drag_finish (context, TRUE, FALSE, time_);
+			return;
+		}
+
+		data = (const gchar *) gtk_selection_data_get_data (selection);
+		empathy_send_file_from_uri_list (contact, data);
+
+		gtk_drag_finish (context, TRUE, FALSE, time_);
+	}
 	else if (info == DND_DRAG_TYPE_TAB) {
 		EmpathyChat        **chat;
 		EmpathyChatWindow   *old_window = NULL;
@@ -1465,6 +1650,13 @@ chat_window_finalize (GObject *object)
 			}
 	}
 
+	if (priv->contact_targets) {
+		gtk_target_list_unref (priv->contact_targets);
+	}
+	if (priv->file_targets) {
+		gtk_target_list_unref (priv->file_targets);
+	}
+
 	chat_windows = g_list_remove (chat_windows, window);
 	gtk_widget_destroy (priv->dialog);
 
@@ -1576,6 +1768,12 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 
 	g_object_unref (accel_group);
 
+	/* Set up drag target lists */
+	priv->contact_targets = gtk_target_list_new (drag_types_dest_contact,
+						     G_N_ELEMENTS (drag_types_dest_contact));
+	priv->file_targets = gtk_target_list_new (drag_types_dest_file,
+						  G_N_ELEMENTS (drag_types_dest_file));
+
 	/* Set up smiley menu */
 	smiley_manager = empathy_smiley_manager_dup_singleton ();
 	submenu = empathy_smiley_menu_new (smiley_manager,
@@ -1616,8 +1814,13 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			   GTK_DEST_DEFAULT_ALL,
 			   drag_types_dest,
 			   G_N_ELEMENTS (drag_types_dest),
-			   GDK_ACTION_MOVE);
+			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
+	/* connect_after to allow GtkNotebook's built-in tab switching */
+	g_signal_connect_after (priv->notebook,
+				"drag-motion",
+				G_CALLBACK (chat_window_drag_motion),
+				window);
 	g_signal_connect (priv->notebook,
 			  "drag-data-received",
 			  G_CALLBACK (chat_window_drag_data_received),
@@ -1645,7 +1848,7 @@ empathy_chat_window_new (void)
  * be added.
  */
 EmpathyChatWindow *
-empathy_chat_window_get_default (void)
+empathy_chat_window_get_default (gboolean room)
 {
 	GList    *l;
 	gboolean  separate_windows = TRUE;
@@ -1660,13 +1863,26 @@ empathy_chat_window_get_default (void)
 	}
 
 	for (l = chat_windows; l; l = l->next) {
+		EmpathyChatWindowPriv *priv;
 		EmpathyChatWindow *chat_window;
 		GtkWidget         *dialog;
 
 		chat_window = l->data;
+		priv = GET_PRIV (chat_window);
 
 		dialog = empathy_chat_window_get_dialog (chat_window);
 		if (empathy_window_get_is_visible (GTK_WINDOW (dialog))) {
+			guint nb_rooms, nb_private;
+			empathy_chat_window_get_nb_chats (chat_window, &nb_rooms, &nb_private);
+
+			/* Skip the window if there aren't any rooms in it */
+			if (room && nb_rooms == 0)
+				continue;
+
+			/* Skip the window if there aren't any 1-1 chats in it */
+			if (!room && nb_private == 0)
+				continue;
+
 			/* Found a visible window on this desktop */
 			return chat_window;
 		}
@@ -1716,6 +1932,9 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 		if (separate_windows) {
 			name = empathy_chat_get_id (chat);
 		}
+		else if (empathy_chat_is_room (chat)) {
+			name = "room-window";
+		}
 
 		empathy_geometry_bind (GTK_WINDOW (priv->dialog), name);
 	}
@@ -1883,7 +2102,7 @@ empathy_chat_window_present_chat (EmpathyChat *chat)
 
 	/* If the chat has no window, create one */
 	if (window == NULL) {
-		window = empathy_chat_window_get_default ();
+		window = empathy_chat_window_get_default (empathy_chat_is_room (chat));
 		if (!window) {
 			window = empathy_chat_window_new ();
 		}
@@ -1898,3 +2117,24 @@ empathy_chat_window_present_chat (EmpathyChat *chat)
  	gtk_widget_grab_focus (chat->input_text_view);
 }
 
+void
+empathy_chat_window_get_nb_chats (EmpathyChatWindow *self,
+			       guint *nb_rooms,
+			       guint *nb_private)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (self);
+	GList *l;
+	guint _nb_rooms = 0, _nb_private = 0;
+
+	for (l = priv->chats; l != NULL; l = g_list_next (l)) {
+		if (empathy_chat_is_room (EMPATHY_CHAT (l->data)))
+			_nb_rooms++;
+		else
+			_nb_private++;
+	}
+
+	if (nb_rooms != NULL)
+		*nb_rooms = _nb_rooms;
+	if (nb_private != NULL)
+		*nb_private = _nb_private;
+}
